import { useEditor, EditorContent } from '@tiptap/react';
import Highlight from '@tiptap/extension-highlight';
import TextAlign from '@tiptap/extension-text-align';
import Blockquote from '@tiptap/extension-blockquote';
import Image from '@tiptap/extension-image';
import StarterKit from '@tiptap/starter-kit';
import { Bold, Italic, List, ListOrdered, Quote, Redo, Undo, AlignLeft, AlignCenter, Plus, Minus, Text } from 'lucide-react';
import { useEffect, useState } from "react";


interface TipTapEditorProps {
  initialContent: string;
  onContentChange: (content: string) => void;
}

/*
const ResizableImage = Image.extend({
  addAttributes() {
    return {
      ...this.parent?.(),
      src: { default: null, },
      alt: { default: null, },
      title: { default: null, },
      class: { default: 'max-w-full h-auto', },
      width: {
        default: 'auto',
        parseHTML: (element) => element.getAttribute('width') || 'auto',
        renderHTML: (attributes) => ({
          width: attributes.width,
          class: selectedImage === attributes.src ? 'selected-image' : '',
        }),
      },
      height: {
        default: 'auto',
        parseHTML: (element) => element.getAttribute('height') || 'auto',
        renderHTML: (attributes) => ({
          height: attributes.height,
        }),
      },
    };
  },
});
*/

export function TipTapEditor({ initialContent, onContentChange }: TipTapEditorProps) {
  const [selectedImage, setSelectedImage] = useState<string | null>(null);

  const ResizableImage = Image.extend({
    addAttributes() {
      return {
        ...this.parent?.(),
        src: { default: null, },
        alt: { default: null, },
        title: { default: null, },
        class: { default: 'max-w-full h-auto', },
        width: {
          default: 'auto',
          parseHTML: (element) => element.getAttribute('width') || 'auto',
          renderHTML: (attributes) => ({
            width: attributes.width,
            class: selectedImage === attributes.src ? 'selected-image' : '',
          }),
        },
        height: {
          default: 'auto',
          parseHTML: (element) => element.getAttribute('height') || 'auto',
          renderHTML: (attributes) => ({
            height: attributes.height,
          }),
        },
      };
    },
  });

  const editor = useEditor({
    extensions: [
      StarterKit.configure({
        blockquote: false, // Отключаем дефолтный
      }),
      Blockquote.configure({
        HTMLAttributes: {
          class: 'border-l-4 border-gray-300 pl-4 italic',
        },
      }),
      TextAlign.configure({
        types: ['heading', 'paragraph'],
      }),
      ResizableImage,
      Highlight,
    ],
    content: initialContent || '',
    editorProps: {
      attributes: {
        class: 'prose prose-lg focus:outline-none min-h-[300px] max-w-none',
      },
    },
    onUpdate: ({ editor }) => {
      const { from, to } = editor.state.selection;
      let foundImage = false;

      editor.state.doc.nodesBetween(from, to, (node) => {
        if (node.type.name === 'image') {
          setSelectedImage(node.attrs.src);
          editor.commands.updateAttributes('image', { class: 'selected-image' });
          foundImage = true;
        }
      });

      if (!foundImage) {
        setSelectedImage(null);
      }

      onContentChange(editor.getHTML());
    },
  });

  // Обновление контента при изменении initialContent
  useEffect(() => {
    if (editor && editor.getHTML() !== initialContent) {
      editor.commands.setContent(initialContent);
    }
  }, [initialContent, editor]);

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (!editor?.view.dom.contains(event.target as Node)) {
        setSelectedImage(null);
      }
    };

    document.addEventListener('mousedown', handleClickOutside);
    return () => document.removeEventListener('mousedown', handleClickOutside);
  }, [editor]);


  if (!editor) return null;

  const resizeImage = (delta: number) => {
    if (!editor.isActive('image')) return;

    const attrs = editor?.getAttributes('image');
    const newWidth = Math.max((parseInt(attrs?.width, 10) || 100) + delta, 50);

    editor.chain().focus().updateAttributes('image', {width: `${newWidth}px`}).run();
  };

  return (
    <div className="border rounded-lg overflow-hidden">
      <div className="border-b bg-gray-50 px-4 py-2">
        <div className="flex items-center space-x-4">
          <button
              type="button"
              onClick={() => editor?.chain().focus().toggleBold().run()}
              className={`p-1 rounded hover:bg-gray-200 ${
                  editor?.isActive('bold') ? 'bg-gray-200' : ''
              }`}
          >
            <Bold size={18} />
          </button>
          <button
              type="button"
              onClick={() => {
                const { state, dispatch } = editor.view;
                const tr = state.tr;
                let inSentence = false;
                let isNewParagraph = false;

                state.doc.descendants((node, pos) => {
                  if (node.type.name === 'paragraph') {
                    isNewParagraph = true;
                    return true;
                  }

                  if (node.isText) {
                    const text = node.text || "";
                    const currentPos = pos;
                    let wordStart = -1;
                    let hasUpperCase = false;
                    let sentenceStart = true;

                    // Сброс контекста для нового параграфа
                    if (isNewParagraph) {
                      inSentence = false;
                      sentenceStart = true;
                      isNewParagraph = false;
                    }

                    for (let i = 0; i < text.length; i++) {
                      const char = text[i];

                      // Обновление статуса предложения при обнаружении пунктуации
                      if (/[.!?]/.test(char) && (i === text.length - 1 || /\s/.test(text[i + 1]))) {
                        sentenceStart = true;
                        inSentence = false;
                      }

                      if (wordStart === -1 && /\S/.test(char)) {
                        wordStart = i;
                        hasUpperCase = /^[A-ZА-ЯЁ]/u.test(char);

                        // Первое слово в предложении
                        if (sentenceStart && hasUpperCase) {
                          sentenceStart = false;
                          inSentence = true;
                          continue; // Пропускаем выделение
                        }
                      }

                      if (wordStart !== -1 && (/\s/.test(char) || i === text.length - 1)) {
                        const wordEnd = char === ' ' ? i : i + 1;

                        // Выделяем только слова внутри предложения, не первые
                        if (hasUpperCase && inSentence && !sentenceStart) {
                          tr.addMark(
                              currentPos + wordStart,
                              currentPos + wordEnd,
                              state.schema.marks.bold.create()
                          );
                        }

                        wordStart = -1;
                        hasUpperCase = false;
                      }
                    }
                  }
                  return true;
                });

                dispatch(tr);
              }}
              className="p-1 rounded hover:bg-gray-200"
          >
            <Text size={18} />
          </button>
          <button
              type="button"
              onClick={() => editor?.chain().focus().toggleItalic().run()}
              className={`p-1 rounded hover:bg-gray-200 ${
                  editor?.isActive('italic') ? 'bg-gray-200' : ''
              }`}
          >
            <Italic size={18} />
          </button>
          <button
              type="button"
              onClick={() => editor?.chain().focus().setParagraph().run()}
              className={`p-1 rounded hover:bg-gray-200 ${editor?.isActive('paragraph') ? 'bg-gray-200' : ''}`}
          >
            <AlignLeft size={18} />
          </button>
          <button
              type="button"
              onClick={() => editor?.chain().focus().undo().run()}
              disabled={!editor?.can().chain().focus().undo().run()}
              className="p-1 rounded hover:bg-gray-200 disabled:opacity-50"
          >
            <Undo size={18} />
          </button>
          <button
              type="button"
              onClick={() => editor?.chain().focus().redo().run()}
              disabled={!editor?.can().chain().focus().redo().run()}
              className="p-1 rounded hover:bg-gray-200 disabled:opacity-50"
          >
            <Redo size={18} />
          </button>
          <button
              type="button"
              onClick={() => editor?.chain().focus().toggleBulletList().run()}
              className={`p-1 rounded hover:bg-gray-200 ${
                  editor?.isActive('bulletList') ? 'bg-gray-200' : ''
              }`}
          >
            <List size={18} />
          </button>
          <button
              type="button"
              onClick={() => editor?.chain().focus().toggleOrderedList().run()}
              className={`p-1 rounded hover:bg-gray-200 ${
                  editor?.isActive('orderedList') ? 'bg-gray-200' : ''
              }`}
          >
            <ListOrdered size={18} />
          </button>
          <button
              type="button"
              onClick={() => {
              editor?.chain().focus().toggleBlockquote().run()}}
              className={`p-1 rounded hover:bg-gray-200 ${
                  editor?.isActive('blockquote') ? 'bg-gray-200' : ''
              }`}
          >
            <Quote size={18} />
          </button>
          <button
              type="button"
              onClick={() => {
              editor?.chain().focus().setTextAlign('center').run()}}
              className={`p-1 rounded hover:bg-gray-200 ${
                  editor?.getAttributes('textAlign')?.textAlign === 'center' ? 'bg-gray-200' : ''
              }`}
          >
            <AlignCenter size={18} />
          </button>
          <button
              type="button"
              onClick={() => {
                const url = prompt('Введите URL изображения');
                if (url) {
                  editor?.chain().focus().setImage({ src: url }).run();
                }
              }}
          >
            🖼️ Фото
          </button>
          <button
              type="button"
              onClick={() => editor?.chain().focus().setTextAlign('center').run()}
              className={`p-1 rounded hover:bg-gray-200 ${
                  editor?.getAttributes('textAlign')?.textAlign === 'center' ? 'bg-gray-200' : ''
              }`}
          >
            🏛 Центр
          </button>
          <button
              type="button"
              onClick={() => resizeImage(20)} className="p-1 rounded hover:bg-gray-200">
            <Plus size={18} />
          </button>
          <button
              type="button"
              onClick={() => resizeImage(-20)} className="p-1 rounded hover:bg-gray-200">
            <Minus size={18} />
          </button>
        </div>
      </div>
      <div className="p-4">
        <EditorContent editor={editor} />
      </div>
    </div>
  );
}